#!/usr/bin/env python3
import requests
import re,os
from multiprocessing.pool import ThreadPool
from os import path
from hashlib import md5
import shelve
__version__='1.1'
class Wget:
    def __init__(self,url,**kargs):
        url0=url
        url=url.split('?')[0]
        self.n=0
        dbname=md5(url.encode()).hexdigest()[:7]+'.pydb'
        record_db=shelve.open(dbname)
        self.filetype=kargs.get('filetype','.jpg')
        self.session=self.setbrowser()
        self.srcpro=None
        self.rule=kargs.get('rule',list())
        self.re_rule=kargs.get('re_rule',list())
        self.asyncThread=ThreadPool(4)
        self.htm=url.split('/')[2]
        if not path.isdir(self.htm): os.makedirs(self.htm)
        self.auto=kargs.get('auto',True)
        print(self.htm)

        self.target_div=kargs.get('div',None)

        self.rule_list=[re.sub('[^A-Za-z]','', url)]
        [self.rule_list.append(re.sub('[^A-Za-z]','', i)) for i in self.rule]
        self.rule_list=list(set(self.rule_list))
        self.rule_dir=[path.dirname(url)]
        [self.rule_dir.append(path.dirname(i)) for i in self.rule]
        self.rule_dir=list(set(self.rule_dir))

        self.re_rule=[re.compile(i) for i in self.re_rule]
        url=url0
        print(self.re_rule,'\n',self.rule_dir,'\n',self.rule_list)
        self.autofind(url)
        try:
            halt=record_db.get('halt',False)
            if halt == True:
                self.href=record_db.get('href',[(url,{})])
                self.pagedb = record_db.get('pagedb',set())
                self.srcdb = record_db.get('srcdb',set())
                record_db['halt']=False
            else:
                self.href=[(url,{})]
                self.pagedb=set()
                self.srcdb=set()
            self.main()
            record_db.close()
            if path.isfile(dbname): os.remove(dbname)
        except:
            print('the program is halted!')
            record_db['halt']=True
            record_db['srcdb']=self.srcdb
            record_db['pagedb']=self.pagedb
            record_db['href']=[i for i in self.href if i!=None]
        self.asyncThread.close()
        self.asyncThread.join()
    def my_hash(self,x):
        return int(md5(x.encode()).hexdigest()[:8],16)
    def setbrowser(self):
        headers='''User-Agent: Mozilla/5.0 (X11; Fedora; Linux x86_64; rv:48.0) Gecko/20100101 Firefox/48.0
DNT: 1
Connection: keep-alive
Upgrade-Insecure-Requests: 1'''
        headers=headers.split('\n')
        d=dict()
        for i in headers:
            n=i.find(':')
            d[i[:n]]=i[n+1:]
        headers=d
        s=requests.session()
        s.auth = ('user', 'pass')
        # s.headers=headers
        return s
    def autofind(self,url):
        print('Autofind the label of the target picture')
        volume=0
        x,y=self.Analyze(url)
        r=set()
        for i in y:
            s=str(i[1])
            if s in r:continue
            r.add(s)
            try:
                t=len(self.session.get(i[0],timeout=30).content)
            except:
                t=0
            if volume<t:
                volume=t
                tt=i[1]
        if self.target_div is not None:
            self.srcpro=(self.target_div,)
        else:
            self.srcpro=tuple(tt.items())
        print(self.srcpro)
    def main(self):
        '''主循环函数，控制寻找页面以及查找页面资源链接，主要起控制作用'''
        def run(u):
            print('Analyse the html ',u)
            hr,src=self.Analyze(u)

            for i in src:
                if self.UsefulSrc0(*i):
                    self.asyncThread.apply_async(self.Download,(i[0],u))
                    # self.Download(i[0],u)
            self.pagedb.add(self.my_hash(u))
            for i in hr:
                if self.UsefulHtml0(*i):
                    self.href.append(i)
        while True:
            ii=0
            n=len(self.href)
            while ii<n:
                if self.UsefulHtml0(*self.href[ii]):
                    run(self.href[ii][0])
                self.href[ii]=None
                ii+=1
            self.href=[i for i in self.href if i!=None]
            if len(self.href)==0:break
    def DivSplit(self,s):
        '''将一个html页面分成多个div块，主要通过寻找div标签的位置，返回一个记录了div块所在位置以及各个块的名字的链表'''
        a=[]
        cp=re.compile('< *div[^><]*>',re.I)
        [a.append((-1,i.span())) for i in cp.finditer(s)]
        b=[]
        for i,j in a:
            if i==1:
                b.append((i,j[0]))
            else:
                t=s[j[0]:j[1]]
                n=re.findall('id *= *"[^"]*"|class *= *"[^"]*"|align *= *"[^"]*"', t)
                d=dict([i.replace('"','').split('=') for i in n])
                b.append((i,j[0],d))
        b.sort(key=lambda x:x[1])
        return b

    def Download(self,url,purl):
        '''下载url'''
        tf=re.sub('[^\w\d]','', purl)
        filename=path.join(self.htm,tf[-min(len(tf),12):]+md5(url.encode()).hexdigest()[:3]+self.filetype)

        if not path.isfile(filename):
            print('Downloading ',url)
            x=self.session.get(url,timeout=30)
            t=open(filename,'wb').write(x.content)
        else:
            print('file already exist')
        self.srcdb.add(self.my_hash(url))
        return filename
    def UsefulHtml0(self,url,pro):
        '''判断一个页面的url是否是有用的'''
        if self.my_hash(url) in self.pagedb:return False
        if self.re_rule:
            for i in self.re_rule_list:
                if i.search(url):return True
        if not self.rule:
            if not re.search(self.htm, url):return False
        t= re.sub('[^A-Za-z]','', url)
        d=path.dirname(url)
        if t in self.rule_list:return True
        if d in self.rule_dir:return True
        if self.auto:
            return False
        return self.UsefulHtml(url,pro)
    def UsefulHtml(self,url,pro):
        '''判断一个页面的url是否是 有用的，这个函数可以在不同的环境中重写，其中pro是该链接所在div的属性'''
        return True
    def UsefulSrc0(self,url,pro):
        if self.my_hash(url) in self.srcdb:return False
        if self.auto:
            for k,v in self.srcpro:
                if v.isdigit():continue
                if pro.get(k)!=v:return False
        return self.UsefulSrc(url,pro)
    def UsefulSrc(self,url,pro):
        return True
    def correct_url(self,s,website,webdir):
        if s[0]=='/':return website+s
        elif s=='#':return ''
        elif s.find('http')!=-1:return s
        else: return webdir+s
    def Analyze(self,url):
        '''返回 href 和 src的链接，返回值为一个二元tuple'''
        s=self.session.get(url,timeout=30).text
        divs=self.DivSplit(s)
        href=[]
        src=[]
        split_url=url.split('/')
        website='/'.join(split_url[:3])
        webdir='/'.join(split_url[:-1])+'/'
        for i in re.finditer(' *(href|src) *=["\' ]*[^ )("\';\+>}]+', s):
            div=self.FindDiv(divs, i.span()[0])
            j=i.group()
            j=re.sub('["\' \\\\]', '', j) #针对某些网站将url写在javascript中，用到了转义符\
            if j[0]=='h':
                j=j.replace('href=', '')
                j=self.correct_url(j,website,webdir)
                if len(j)==0:continue
                href.append((j,div))
            if j[0]=='s':
                j=j.replace('src=', '')
                if j.find(self.filetype)==-1:continue
                div=self.FindDiv(divs, i.span()[0])
                j=self.correct_url(j,website,webdir)
                if len(j)==0:continue
                src.append((j,div))
        return href,src
    def FindDiv(self,divs,pos):
        a,b=0,len(divs)
        if b==0:
            return {'id':'nodivs'}
        if pos>divs[-1][1]:return divs[-1][2]
        while b-a>1:
            t=int((a+b)/2)
            p0=divs[t][1]
            if pos>p0:a=t
            else:b=t
        return divs[a][2]

if __name__=='__main__':
    pass
    # Wget('https://www.umei.cc/p/gaoqing/gangtai/26074.htm')


#!/usr/bin/env python3
import MySQLdb as _pl
__version__='1.1.1'
class mydb(object):
    def __init__(self,user,passwd,host='127.0.0.1',db=None,port=3306):
        self.conn=_pl.connect(host=host,user=user,passwd=passwd,db=db,port=port,charset='utf8')
        self.cur=self.conn.cursor()
        self.conn.autocommit(1)
        self.db=db
    def select(self,table,columns=('*',),where='1=1'):
        c=','.join(columns)
        s='select %s from %s where %s;' % (c,table,where)
        try:
            c=self.cur.execute(s)
        except Exception as err:
            print(s,err)
            return False
        c=self.cur.fetchall()
        return c
    def insertmany(self,table,dl):
        a=[]
        dargvs=dl[0]
        t=','.join(['`'+i+'`' for i in dargvs.keys()])
        for dargvs in dl:
            v=['"'+i+'"' if isinstance(i,str) else str(i) for i in dargvs.values()]
            v=','.join(v)
            v='('+v+')'
            a.append(v)
        a=','.join(a)
        s='insert into '+table+'('+t+') values '+a+';'
        try:
            self.cur.execute(s)
        except:
            return False
        return None
    def insert(self,table,**dargvs):
        t=','.join(['`'+i+'`' for i in dargvs.keys()])
        v=['"'+i+'"' if isinstance(i,str) else str(i) for i in dargvs.values()]
        v=','.join(v)
        v='('+v+')'
        s='insert into '+table+'('+t+') values'+v
        try:
            self.cur.execute(s)
        except:
            return False
        return None
    def execute(self,s):
        try:
            c=self.cur.execute(s)
        except:
            print('Execute wrong sql :',s)
            return False
        c=self.cur.fetchall()
        return c
    def delete(self,table,where):
        self.cur.execute('delete from '+table+' where '+where)
    def update(self,table,where,**kargs):
        s=''
        for i,j in kargs.items():
            s+='`{0}`="{1}",'.format(i,j)
        s='update {table} set {com} where {where}'.format(table=table,com=s[:-1],where=where)
        c=self.cur.execute(s)
        return c
    def CreateTable(self,table,content,coding='ENGINE=InnoDB DEFAULT CHARSET=utf8'):
        self.cur.execute('CREATE TABLE `'+table+'` '+content+' '+coding)
    def AddColumns(self,table,**kargs):
        for key,value in kargs.items():
            self.cur.execute('alter table `'+table+'` add column `'+key+'` '+value)
    def DropColumns(self,table,*targs):
        for i in targs:
            self.cur.execute('alter table `'+table+'` drop column `'+i+'`')
    def DropTable(self,table):
        self.cur.execute('DROP TABLE `%s`;' % (table,))
    def QueryRom(self,db,table=None):
        ''' SELECT sum(DATA_LENGTH)+sum(INDEX_LENGTH) FROM information_schema.TAbles where TABLE_SCHEMA='stock';'''
        x=self.cur.execute("SELECT sum(DATA_LENGTH)+sum(INDEX_LENGTH) FROM information_schema.TAbles where TABLE_SCHEMA='%s';" % (db,))
        return self.cur.fetchall()[0][0]/(1024**3)
    def __del__(self):
        self.cur.close()
        self.conn.close()


#!/usr/bin/env python3
import os
from os.path import getsize
import os.path as path
import shutil
version='0.1'
def _DeleteFiles(a):
    return
    if len(a)==0:
        print('No file need to be deleted')
        return
    else:
        for i in a:print(i)
        x=input('Do you want to delete these files?[Y/N]')
    if x.strip().lower()!='y':
        print('No thing to do.')
        return
    else:
        for i in a:os.remove(i)
def issame(file1,file2,index=0):
    xf=open(file1,'rb')
    yf=open(file2,'rb')
    x=xf.read()
    y=yf.read()
    xf.close()
    yf.close()
    if len(x)!=len(y):return False
    if x==y:
        return True
    else:
        if index==1:
            num=len(x)
            i=0
            k=0
            while i<num:
                if x[i]!=y[i]:k+=1
                i+=1
            k=float(k)/num
            if k<1e-5:
                return True
        return False  
def Bianli(dirname):
    y=os.walk(dirname)
    x=[]
    for i in y:
        x.extend([i[0]+os.sep+j for j in i[2]])
    return x           
def DeleteSameFiles(dirname,floor=0,index=0,pause=False):
    delfiles=[]
    if floor!=0:x=Bianli(dirname)
    else:x=[dirname+os.sep+i for i in os.listdir(dirname)]
    zong=float(len(x))
    x=[(i,getsize(i)) for i in x if path.isfile(i)]
    x.sort(key=lambda x:x[1])
    kk0=0
    for i in range(len(x)-1):
        kk=i/zong
        if kk-kk0>0.05:
            print(u'进度:%-5.3f%%' % (kk*100))
            kk0=kk
        if x[i][1]==x[i+1][1]:
            if issame(x[i][0],x[i+1][0],index):
                if x[i+1][0]<x[i][0]:
                    t=x[i]
                    x[i]=x[i+1]
                    x[i+1]=t
                delfiles.append(x[i][0])
    _DeleteFiles(delfiles)
def Operate(dirname,handle=None,floor=0):
    x=os.walk(dirname)
    if floor==0:x=[next(x)]
    for i in x:
        for j in i[2]:
            handle(i[0],j,i[0]+os.sep+j,getsize(i[0]+os.sep+j))

if __name__=='__main__':
    Operate('/home/yxs/D/music',handle=kk,floor=1)
    # x=time.time()
    # DeleteSameFiles('/home/yxs/E/学习/书/Calibre书库',floor=1)
    # print(time.time()-x)


#!/usr/bin/env python3
import numpy as np
from os.path import getsize
__version__='1.1'
class _zone_data(dict):
    def rename(self,old,new):
        self[old]=self[new]
        self.pop(old)
    def set_data(self,names,values,attribute):
        self.names=names
        self.data=values
        self.attribute=attribute
        for i,v in zip(names,values):
            self[i]=v
    def __getitem__(self,k):
        if isinstance(k,str):
            return super().__getitem__(k)
        else:
            if self.attribute == 'fluent_prof':
                return self.data[:,k]
class read(dict):
    def __init__(self,filename,filetype=None,**kargs):
        self.fp=open(filename,'r')
        self.default_filetypes={'prof':'fluent_prof','dat':'tecplot_dat','out':'fluent_residual_out',
        'out2':'fluent_monitor_out','csv':'csv'}
        self.data=None
        if filetype is None:
            key=filename.split('.')[-1].lower()
            if key=='out':key=self.__recognize_out(self.fp)
            self.filetype=self.default_filetypes[key]
        else:
            self.filetype=filetype
        self.filesize=getsize(filename)
        if self.filetype=='tecplot_dat':
            self._read_dat()
        elif self.filetype=='fluent_prof':
            self._read_prof()
        elif self.filetype=='fluent_residual_out':
            self._read_out()
        elif self.filetype=='fluent_monitor_out':
            self.__read_out2()
        elif self.filetype=='csv':
            self.__read_csv(filename)
        self.fp.close()
    def __read_csv(self,filename):
        title=self.fp.readline()
        tmp=np.loadtxt(self.fp,dtype='float64',delimiter=',')
        title=title.strip().split(',')
        for i,j in enumerate(title):
            self[j]=tmp[:,i]
        self.data=tmp
    def __recognize_out(self,fp):
        fp.readline()
        t=fp.readline()
        t=t.split()
        key='out'
        if t:
            if t[0]=='"Iteration"':
                key='out2'
        fp.seek(0,0)
        return key
    def __read_out2(self):
        self.fp.readline()
        t=self.fp.readline()
        t=t.lstrip()[11:].strip()[1:-1]
        d=self.fp.read().encode().strip()
        d=d.split(b'\n')
        d=[tuple(i.split()) for i in d]
        x=np.array(d,dtype=np.dtype({'names':["Iteration",t],'formats':['int32','float64']}))
        self["Iteration"]=x['Iteration']
        self[t]=x[t]
        self.data=x
    def _read_out(self):#fluent residual file
        items=[]
        items_n=0
        data=[]
        iter_pre='0'
        time_index=False
        for i in self.fp:
            if i[:7]=='  iter ':
                if items_n!=0:continue
                j=i.strip().split()
                items.extend(j)
                if items[-1]=='time/iter':
                    items.pop()
                    items.extend(('time','iter_step'))
                    time_index=True
                items_n=len(items)
            if items_n==0:continue
            else:
                j=i.split()
                if len(j)==items_n:
                    if j[0].isdigit():
                        if j[0]==iter_pre:continue
                        iter_pre=j[0]
                        if time_index:j.pop(-2)
                        data.append(tuple(j))
        if time_index:items.pop(-2)
        a=np.array(data,dtype=np.dtype({'names':items,'formats':['i']+['f']*(len(items)-2)+['i']}))
        for i,k in enumerate(items):
            self[k]=a[k]
        self.data=a
    def _read_prof(self):
        fp=self.fp
        d=fp.read()
        d=d.replace('\r','')
        d=d.split('((')
        d.pop(0)
        data=[]
        def read(x):
            x=x.split('(')
            title=x[0].split()[0]
            x.pop(0)
            data=[]
            name=[]
            ii=0
            for i in x:
                c=i.split('\n')
                ii+=1
                name.append(c[0])
                data.append(c[1:-2])
            data[-1].pop()
            values=np.array(data,dtype='float32')
            if len(values)!=len(name):return False
            t=_zone_data()
            t.set_data(name,values,self.filetype)
            return title,t
        for i in d:
            k,v=read(i)
            self[k]=v
    def parse_zone(self):
        fp=self.fp
        t=fp.readline().rstrip()
        if t[:4]!='ZONE':
            return False
            raise EOFError('wrong file type!')
        title=t[t.find('"')+1:-1].strip()
        n=title.find('Step ')
        if n!=-1:title=title[:n].strip()
        attr={}
        while True:
            t=fp.readline().strip()
            if t[:2]=='DT':break
            t=t.replace(',',' ').split()
            for i in t:
                k,v=i.split('=')
                if v.isdigit():v=int(v)
                attr[k]=v
        if t.find('SINGLE')==-1:
            raise EOFError('file type error')
        t=t[t.find('(')+1:t.find(')')].strip().split()
        attr['DT']=tuple(t)
        return title,attr
    def _read_dat(self):
        fp=self.fp
        t=fp.readline().rstrip()
        self.title=t[t.find('"')+1:-1]
        t=fp.readline().rstrip()
        self.variables=[t[t.find('"')+1:-1]]
        while True:
            i=fp.readline()
            t=i.strip()
            if t[0]!='"':break
            pos=fp.tell()
            t=t[t.find('"')+1:-1]
            n=t.find('Dataset:')
            if n!=-1:
                t=t[:n].strip()
            self.variables.append(t)
        fp.seek(pos)
        self.variables=tuple(self.variables)
        a=[]
        while True:
            t=self.parse_zone()
            if t is False:
                break
            a.append(t)
            t=self._read_data(t)
        self.zones={i[0]:i[1] for i in a}
    def _read_data(self,a):
        fp=self.fp
        title,attribute=a
        zo=_zone_data()
        self[title]=zo
        if 'Nodes' in attribute and 'Elements' in attribute:
            n=attribute['Nodes']
            # self.now_keys={k:d[i*n:i*n+n] for i,k in enumerate(self.parent.variables)}
            m=attribute['Elements']
            nn=n//5
            if n%5>0:nn=int(nn+1)
            for j,key in enumerate(self.variables):
                ii=0
                s=[]
                for i in fp:
                    s.append(i)
                    ii+=1
                    if ii==nn:break
                if attribute['DT'][j]=='SINGLE':
                    ddtype='float32'
                else:
                    raise EOFError('DT type wrong!')
                zo[key]=np.array(''.join(s).split(),dtype=ddtype)
            s=[]
            for i in fp:
                s.append(i.split())
                m-=1
                if m==0:break
            zo['ELEMENTS']=np.array(s,dtype='int32')
        return True
    def __getitem__(self,k):
        if isinstance(k,str):
            return super().__getitem__(k)
        else:return self.data[k]
    def rename(self,old,new):
        self[new]=self[old]
        self.pop(old)
    def write(self,filename):
        write(self,filename)
class write:
    def __init__(self,data,filename,filetype=None):
        default_filetypes={'prof':'fluent_prof','dat':'tecplot_dat','out':'fluent_residual_out',
        'out2':'fluent_monitor_out'}
        if filetype is None:
            filetype=data.filetype
        if filetype=='fluent_prof':
            self.__write_prof(data,filename)
        elif filetype=='tecplot_dat':
            self.__write_dat(data,filename)
        elif filetype=='csv':
            self.__write_csv(data,filename)
        elif filetype=='fluent_monitor_out':
            self.__write_out2(data,filename)
        elif filetype=='fluent_residual_out':
            self.__write_out(data,filename)
        else:
            raise EOFError('file type error!')
    def __write_out(self,data,filename):
        fp=open(filename,'w')
        self.__write_delimiter(data,fp,'  ',title_format='',specified_format=' %d',specified_titles=['iter'],other_format='%.8e')
        fp.close()
    def __write_out2(self,data,filename):
        fp=open(filename,'w')
        value=[i for i in data.keys() if i!='Iteration'][0]
        fp.write('"Convergence history of %s"\n' % value)
        self.__write_delimiter(data,fp,' ',title_format='"',specified_format='%d',specified_titles=['Iteration'])
        fp.close()
    def __write_csv(self,data,filename):
        fp=open(filename,'w')
        self.__write_delimiter(data,fp,',')
        fp.close()
    def __write_delimiter(self,data,fp,delimiter,title_format='',specified_format='',specified_titles=[],other_format='%.15e'):
        other_titles=[i for i in data.keys() if i not in specified_titles]
        title=specified_titles+other_titles
        title_w=[title_format+i+title_format for i in title]
        fp.write(delimiter.join(title_w)+'\n')
        s=np.vstack([data[i] for i in title]).T
        data_format=specified_format+delimiter+delimiter.join([other_format]*len(other_titles))+'\n'
        for i in s:
            fp.write(data_format % tuple(i))
    def __write_prof(self,data,filename):
        fp=open(filename,'wb')
        for i in data.keys():
            keys=list(data[i].keys())
            keys.sort()
            keys.sort(key=lambda x:len(x))
            n=len(data[i][keys[0]])
            fs='(('+i+' point '+str(n)+')\n'
            fp.write(fs.encode())
            for k in keys:
                fs='('+k+'\n'
                fp.write(fs.encode())
                [fp.write((str(j)+'\n').encode()) for j in data[i][k]]
                fp.write(')\n'.encode())
            fp.write(')\n'.encode())
    def __write_dat(self,data,filename):
        if 'Nodes' in attribute and 'Elements' in attribute:
            pass
if __name__=='__main__':
    # print(x['wall']['CoordinateX'],x['wall']['CoordinateY'])
    # print(x.keys())
    # print(x['in'].keys())
    # s=time.time()
    # x=open('1.dat','r').read()
    # print(time.time()-s)
    # time.sleep(1)
    # s=time.time()
    # x=open('1.dat','r').read()
    # print(time.time()-s)
    f1='D:\\sst-kw_check\\residual_yxs_0605_ddes_profile.out'
    f2='D:\\sst-kw_check\\surf-mon-17_p5yv.out'
    f3='D:\\sst-kw_check\\6800.prof'
    f4='F:\\test.csv'
    x=read(f1)
    x.write('F:\\test_out1.out')


#!/usr/bin/env python3
from time import strptime,mktime,strftime,localtime,time
from multiprocessing.pool import Pool,ThreadPool
from sys import stdout
__version__='1.4.0'
#modify:2017-02-03
def DaysToDate(x):
    return strftime('%Y-%m-%d',localtime((x-719163)*86400))
def DateToWeekdays(date):
    return (strptime(date, '%Y-%m-%d')[6]+1)%7
def DateToDays(x):
    return int((mktime(strptime(x,'%Y-%m-%d'))+28800)/86400+719163)
def TodayToDate(p=0):
    return DaysToDate(DateToDays(strftime('%Y-%m-%d',localtime()))+p)
#多进程测试
def ProcessesTest(target,args=[],num=None,func=Pool,callback=None):
    if num is None:p=func()
    else:p=func(num)
    if callback is 'auto':
        n=len(args)
        def CallBack(*d,count=[0,0,time(),time()]):
            count[0]+=1
            ii,ii0,start,end=count
            count[3]=time()
            t=((count[3]-start)/ii)*(n-ii)
            print('completed: {0:.3f}%, time remaining {h:.0f}:{m:02.0f}:{s:02.0f}.'.format(ii/n,h=(t//3600),m=(t%3600)//60,s=int(t%60)))
        callback=CallBack
    for i in args:
        p.apply_async(target,args=i,callback=callback)
    p.close()
    p.join()
#多线程测试
def ThreadsTest(target,args=[],num=None,callback=None):
    ProcessesTest(target=target,args=args,num=num,func=ThreadPool,callback=None)
def Record(db,Key,Value='NULL',Type='NULL'):
    # db.execute('INSERT INTO Record (`Type`,`Key`,`Value`) VALUES ({Type},{Key},{Value})  ON DUPLICATE KEY UPDATE Value={Value}'.format(Key=Key,Value=Value,Type=Type))
    db.insert('Record',Type=Type,Value=Value,Key=Key)
def Query(db,Key=None,Value=None,Type=None):
    where=list()
    if Key!=None:
        where.append('Key="'+Key+'"')
    if Value!=None:
        where.append('Value="'+Value+'"')
    if Type!=None:
        where.append('Type="'+Type+'"')
    if len(where)>0:
        where=' and '.join(where)
    else:
        where='1=1'
    x=db.select('Record',('`Type`','`Key`','`Value`'),where=where)
    return x
def Delete(db,Key=None,Value=None,Type=None):
    where=''
    if Key!=None:where+='and `Key`="%s" ' % (Key,)
    if Value!=None:where+='and `Value`="%s" ' % (Value,)
    if Type!=None:where+='and `Type`="%s" ' % (Type,)
    if len(where)==0:
        where='1=1'
    else:where=where[3:]
    db.cur.execute('delete from Record where '+where)
def Code(s,code='utf8'):
    '''输入参数可以是整数、tuple、list、字符串、其中以0x开头的字符串被认为是16进制的字符串，code是编码格式，默认utf8
    输出参数：（编码格式，整数、每个字节十进制整数，16进制的数，编码的结果）如果显示为none则认为无该编码结果'''
    htobyten=lambda h:[int(h[2+i*2:2+i*2+2],16) for i in range(int((len(h)/2)-1))]
    def htos(h):
        by=bytearray(htobyten(h))
        try:
            return by.decode(code)
        except:
            return None
    if isinstance(s,str):
        if len(s)>2 and s[:2]=='0x':
            h=s
            n=int(h,16)
            byten=htobyten(h)
            s=htos(h)
        if (len(s)>2 and s[:2]!='0x') or len(s)<=2:
            x=bytearray(s,code)
            h='0x'+''.join([hex(i)[2:].zfill(2) for i in x])
            n=int(h,16)
            byten=[int(i) for i in x]
    elif isinstance(s, tuple) or isinstance(s, list):
        x=bytearray(s)
        byten=s
        h='0x'+''.join([hex(i)[2:].zfill(2) for i in s])
        try:
            s=x.decode(code)
        except:
            s=None
        n=int(h,16)
    else:
        h=hex(s)
        n=s
        byten=htobyten(h)
        s=htos(h)
    return code,n,byten,h,s
def progress_bar(p,s1='',s2='',length=40):
	s='\r{0}|{1}|{2}'
	n1=int(p*length+0.1)
	n2=length-n1
	m1='#'*n1 
	m2=' '*n2
	stdout.write(s.format(s1,m1+m2,s2))
	stdout.flush()
if __name__=='__main__':
	pass
    # import time
    # for i in range(400):
    # 	progress_bar(i/400)
    # 	time.sleep(0.02)


#!/usr/bin/env python3
import hashlib
__version__='1.0'
def md5(a):
    x=hashlib.md5()
    f=open(a,'rb')
    while a!=b'':
        a=f.read(8388608)
        x.update(a)
    f.close()
    return x.hexdigest()


#!/usr/bin/env python3
import smtplib ,email
import email.mime.multipart as Multipart# import MIMEMultipart  
import email.mime.text as Text # import MIMEText  
import email.mime.base as Base# import MIMEBase  
import os.path  as path
import mimetypes  
import imaplib, string
import time
import poplib
__version__='1.0.1'
class send_email():
    def __init__(self,account,password,Sender=' ',To=' ',smtpServer=None,login=True):
        if smtpServer is not None:
             self.server = smtplib.SMTP(smtpServer)
        if login is True:
            self.login(account=account,password=password)
        self.Sender=Sender
        self.To=To
        self.account=account
    def login(self,account,password):
        smtp=account.split('@')[-1]
        try:
            self.server = smtplib.SMTP("smtp."+smtp)
            self.server.login(account,password) #仅smtp服务器需要验证时  
            return True
        except:
            return False
    def send(self,content,accessory=[],receiver=[],Subject='',To = None ,From = " ",
        Sender=None): 
        if Sender is None:Sender=self.Sender
        if To is None:To=self.To
        # 构造MIMEMultipart对象做为根容器  
        main_msg = Multipart.MIMEMultipart()     
        # 构造MIMEText对象做为邮件显示内容并附加到根容器  
        text_msg = Text.MIMEText(content,_charset="utf-8")  
        main_msg.attach(text_msg)  
              
        # 构造MIMEBase对象做为文件附件内容并附加到根容器  
              
        ## 读入文件内容并格式化 [方式1]－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－  
        def AddAccessory(filename):
            if not path.isfile(filename):return
            data = open(filename, 'rb')  
            ctype,encoding = mimetypes.guess_type(filename)  
            if ctype is None or encoding is not None:  
                ctype = 'application/octet-stream'  
            maintype,subtype = ctype.split('/',1)  
            file_msg = Base.MIMEBase(maintype, subtype)  
            file_msg.set_payload(data.read())  
            data.close( )  
            email.encoders.encode_base64(file_msg)#把附件编码  
            ## 设置附件头  
            basename = path.basename(filename)  
            file_msg.add_header('Content-Disposition','attachment', filename = basename.encode('gbk').decode('gbk'))#修改邮件头  
            main_msg.attach(file_msg)  
        if isinstance(accessory,str):AddAccessory(accessory)
        else:
            for i in accessory:AddAccessory(i)
        # 设置根容器属性  
        From="%s<%s>" % (From,self.account)
        main_msg['From'] = From
        main_msg['To'] = To  
        main_msg['Subject'] = Subject  
        main_msg['Date'] = email.utils.formatdate( )  
        main_msg['Sender']=Sender
        # 得到格式化后的完整文本  
        fullText = main_msg.as_string( )  
              
        # 用smtp发送邮件  
        try:  
            self.server.sendmail(From, receiver, fullText)  
        except:
            return False
        finally:
            return True 
    def __del__(self):
        self.server.quit()
class pop_accept_email:
    def __init__(self,account,password):
        self.account=account
        self.password=password
    def login(self):
        host='pop.'+self.account.split('@')[-1]
        self.pop_conn = poplib.POP3_SSL(host)  
        self.pop_conn.user(self.account)  
        self.pop_conn.pass_(self.password)  
    def accept(self):
        self.login()
        ret = self.pop_conn.stat() 
        n=ret[0]
        if n==0:return None
        #Get messages from server:  
        messages = [self.pop_conn.retr(n) ]
        # Concat message pieces:  
        messages = ["\n".join(mssg[1]) for mssg in messages]  
          
        #Parse message intom an email object:  
        messages = [parser.Parser().parsestr(mssg) for mssg in messages]  
        for message in messages:  
            print(message['Subject'])
            break
        x=n-229
        if x<1:x=1
        for i in range(x,n+1):self.pop_conn.dele(i)
        self.pop_conn.quit()
class imap_accept_email:
    def __init__(self,account,password):
        host='imap.'+account.split('@')[-1]
        self.M = imaplib.IMAP4_SSL(host)  
        self.M.login(account,password)  
    def accept(self,**kargv):
        result, message = self.M.select('INBOX') 
        typ, data = self.M.search(None, 'ALL')  
        for num in string.split(data[0]): 
            typ, data = self.M.fetch(num, '(RFC822)')  
            msg = email.message_from_string(data[0][1])  
            print(msg)
            return msg  
    def __del__(self):
        self.M.logout()
if __name__=='__main__':
    pass


#!/usr/bin/env python3
from setuptools import setup  ,find_packages
install_modules=['bs4','requests','tushare','lxml']
setup(name='yxspkg',   
      version='2.8.8',    
      description='My pypi module',    
      author='Blackso',    
      install_requires=install_modules,
      author_email='1185793964@qq.com',       
      url='http://weibo.com/2320090895/profile',
      packages=find_packages(), 
      classifiers = [
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
)   